(function (global) {
    "use strict";

    global.FRCC = global.FRCC || {};
    global.FRCC.Portal = global.FRCC.Portal || {};
    global.FRCC.Portal.Utilities = (function() {
       
        class EygaApiType {
            static Documents = "eyfrcc_DocumentsAPIURL";
            static Conversion = "eyfrcc_ConversionAPIURL";
            static Identity = "eyfrcc_IdentityAPIURL";
            static Address = "eyfrcc_AddressAPIURL";
        }

        class EygaApiContext {
            constructor(regardingId, regardingType, userId) {
                this.regardingId = regardingId ? regardingId.replace('{', '').replace('}', '') : null;
                this.regardingType = regardingType;
                this.userId = userId ? userId.replace('{','').replace('}','') : null;
            }
        }

        class EygaApiCall {
            constructor(displayName, apiType, apiMethod, httpMethod, xhrFields, contentType, processData) {
                this.displayName = displayName;
                this.apiType = apiType;
                this.apiMethod = apiMethod;
                this.httpMethod = httpMethod;
                this.xhrFields = xhrFields != undefined
                    ? xhrFields : {};
                this.contentType = contentType != undefined
                    ? contentType : "application/json";
                this.processData = processData != undefined
                    ? processData : true;
            }
        }

        class ApiCall {
            static UploadDocument = new EygaApiCall("Upload Document", EygaApiType.Documents, "/upload", "POST", {}, false, false);
            static ConvertDocument = new EygaApiCall("Convert Document to PDF", EygaApiType.Conversion, "/convert", "POST");
            static DownloadDocument = new EygaApiCall("Download Document", EygaApiType.Documents, "/download", "POST", { responseType: 'blob' });
            static DeleteDocument = new EygaApiCall("Delete Download", EygaApiType.Documents, "/delete", "DELETE");
            static GetDocumentsById = new EygaApiCall("Get Documents By Id", EygaApiType.Documents, "/searchbytags", "POST");
            static IdentityRequest = new EygaApiCall("Initiate Identity Request", EygaApiType.Identity, "/requests", "POST");
            static ValidateAddress = new EygaApiCall("Validate Address", EygaApiType.Address, "/validate", "POST");
        }

        const StepNames = Object.freeze({
            RiskIndicator: "Risk Indicator"
        });

        const RiskAssessmentOutcomeType = Object.freeze({
            AddFieldToReviewChecklist: 643_260_000,
            AddFieldToApplication: 643_260_001,
            AddStageToReviewProcess: 643_260_002,
            AdjustReportingFrequency: 643_260_003,
            DenyApplication: 643_260_004,
        });

        var saveSession = function(name, obj) {
            sessionStorage.setItem(name, JSON.stringify(obj));
            return true;
          }
          
        var getSession = function(name) {
            var obj = {};
            var itemFromStorage = sessionStorage.getItem(name);
            if (itemFromStorage !== "undefined") {
              obj = JSON.parse(itemFromStorage);
            }
            return obj;
          }

        // Given the ID of a step on an application, "opens" the div for that step, swapping the images appropriately
        var openAccordian = function (id) {
            var imageId = id + "-image";
            var image = document.getElementById(imageId);
            var elem = document.getElementById(id);
            var accordian = document.getElementById(id + "-accordian")
            if(accordian.classList.contains("disabled")) {
                return;
            }
            if (elem.classList.contains("hidden")) {
                image.src="/Images/down-arrow.svg";
                elem.classList.remove("hidden");
            } else {
                image.src="/Images/side-arrow.svg";
                elem.classList.add("hidden");
            }
        };

        var handleTablePaging = function (currentPage, incrementer, serviceUrl, selectorPrefix, createRowCallback) {
            var previousPage = currentPage;
            currentPage += incrementer;
            if (currentPage < 1) {
                currentPage = 1;
                previousPage = 1;
            }
			
            var loadingIconSelector = `#${selectorPrefix}Loading`;
            $(loadingIconSelector).show();
            var noResultsMessage = `.${selectorPrefix}NoResultsMessage`;
            $(noResultsMessage).hide();
			_disablePageButton(`#${selectorPrefix}Next`);
			_disablePageButton(`#${selectorPrefix}Previous`);
			
            appAjax('Retrieving records...', {
                type: "GET",
                url: serviceUrl.replace("{0}", currentPage),
                contentType: "application/json",
                success: function (response) {
                    var data = JSON.parse(response);
                    if (data.results.length > 0) {
                        $(loadingIconSelector).hide();
                        _refreshTable(data.results, currentPage, selectorPrefix, createRowCallback)
                        _showHidePageButtons(data.results, data.moreRecords);
                        previousPage += incrementer;
                    } else {
                        $(loadingIconSelector).hide();
                        _displayNoResults(selectorPrefix);
                        currentPage = previousPage;
                    }
                    _showHidePageButtons(data.moreRecords, currentPage, selectorPrefix);
                },
                failure: function (res) {
                }
            });
			
			return currentPage;
        };

        var _refreshTable = function(results, currentPage, selectorPrefix, createRowCallback) {
            //clear all rows
            $(`#${selectorPrefix}Table`).find("tbody>tr").remove();
            $(`#${selectorPrefix}PageCount`).text(currentPage);

            //populate table with paginated data
            for(let i = 0; i < results.length; i++) {
                $(`#${selectorPrefix}Table`).find("tbody").append(createRowCallback(results[i]));
            }
        }

        var _displayNoResults = function(selectorPrefix) {
            $(`#${selectorPrefix}Table`).find("tbody>tr").remove();
            $(`.${selectorPrefix}NoResultsMessage`).show();
        }

        var _showHidePageButtons = function(moreRecords, currentPage, selectorPrefix) {
            if (!moreRecords) {
                _disablePageButton(`#${selectorPrefix}Next`);
            } else {
                _enablePageButton(`#${selectorPrefix}Next`);
            }
      
            if (currentPage == 1) {
                _disablePageButton(`#${selectorPrefix}Previous`);
            } else {
                _enablePageButton(`#${selectorPrefix}Previous`);
            }

            if (!moreRecords && currentPage === 1) {
                _disablePageButton(`#${selectorPrefix}PageCount`);
            } else {
                _enablePageButton(`#${selectorPrefix}PageCount`);
            }
        }
        
        var _enablePageButton = function(buttonSelector) {
            $(buttonSelector).parent().show();
        }
      
        var _disablePageButton = function(buttonSelector) {
            $(buttonSelector).parent().hide();
        }

        var formatDate = function(date) {
            if (!date || date == "") {
                return date;
            }
            var i = date.substr(0,date.indexOf(' '));
            var parsedDate = date.substr(date.indexOf(' ')+1); 
            var j = i % 10,
                k = i % 100;
            if (j == 1 && k != 11) {
                return i + parsedDate;
            }
            if (j == 2 && k != 12) {
                return i + parsedDate;
            }
            if (j == 3 && k != 13) {
                return i + parsedDate;
            }
            return i + parsedDate;
        }

		var searchForError = function (nameKey, myArray){
			if (myArray && myArray.Blob) {
				for (var i=0; i < myArray.Blob.length; i++) {
					if (myArray.Blob[i].Name["#text"] === nameKey) {
						return true;
					}
			    }
                if (myArray && !Array.isArray(myArray.Blob)) { 
                    if (myArray.Blob.Name["#text"] === nameKey) {
                        return true;
                    }
                }
            }
            return false;
        };

        var _safeAjax = function safeAjax(ajaxOptions) {
            var deferredAjax = $.Deferred();
            shell.getTokenDeferred().done(function (token) {
                // Add headers for ajax
                if (!ajaxOptions.headers) {
                    $.extend(ajaxOptions, {
                        headers: {
                            "__RequestVerificationToken": token
                        }
                    });
                } else {
                    ajaxOptions.headers["__RequestVerificationToken"] = token;
                }
                $.ajax(ajaxOptions)
                    .done(function (data, textStatus, jqXHR) {
                        validateLoginSession(data, textStatus, jqXHR, deferredAjax.resolve);
                    })
                    .fail(deferredAjax.reject); //ajax
            }).fail(function () {
                deferredAjax.rejectWith(this, arguments); // On token failure pass the token ajax and args
            });
            return deferredAjax.promise();
        };

        var appAjax = function(processingMsg, ajaxOptions) {
            FRCC.Portal.Notification.show(processingMsg);
            return _safeAjax(ajaxOptions)
            .fail(function (response) {
                if (response.responseJSON) {
                    alert("Error: " + response.responseJSON.error.message)
                } else {
                    alert("Error: Web API is not available... ")
                }
            }).always(FRCC.Portal.Notification.hide);
        };

        var _isOldIE = function(userAgent) {
            var msie = userAgent.indexOf('MSIE');
            var trident = userAgent.indexOf('Trident/');
            if (msie > 0 || trident > 0) {
                return true;  // IE 10 or older
            }
            return false; // other browser, IE 11, or Edge
        };

        var showIEWarning = function () {
            if (_isOldIE(navigator.userAgent)) {
                var ieWarning = document.getElementById('ie-banner');
                if (ieWarning) {
                    ieWarning.setAttribute('style', 'display: block;');
                }
            }
        };
        
        var _generateExpirationDate = function(){      
            var date= new Date();
            date.setDate(date.getDate() + 1);  
            return date;
		};

        var _generateCurrentDatePlusOneMinute = function(){
            var date = new Date();    
            var oneMinuteInMilliSeconds = 60000;   
            date = new Date(date.getTime() + oneMinuteInMilliSeconds);
            return date; 	
        };

        var getEnvironmentVariable = function (variableName) {
            return new Promise(function (resolve, reject) {  
                var environmentVariable = window.sessionStorage.getItem(variableName); 
                if(environmentVariable){
                    var currentDatePlusOneMinute = _generateCurrentDatePlusOneMinute();  
                    if(currentDatePlusOneMinute.getTime() <= new Date(environmentVariable.expirationDate).getTime()){       
                        resolve(environmentVariable.value[0].value);	 
				    }
                    else{
                        _getEnvironmentVariableFromCrm(variableName)  
                        .then(value => {
                            window.sessionStorage.setItem(variableName,  JSON.stringify(value));  
                            resolve(value.value[0].value); 
                        })
                        .catch(reason => reject(reason));
                    } 
                }
                else{
                    _getEnvironmentVariableFromCrm(variableName) 
                    .then(value => {      
                        window.sessionStorage.setItem(variableName,  JSON.stringify(value));  
                        resolve(value.value[0].value);
                    })
                    .catch(reason => reject(reason));
                }     
            });
        };

        var _getEnvironmentVariableFromCrm = function(variableName) {
            return new Promise(function (resolve, reject) {
                _safeAjax({
                    type: "GET",
                    url: "/_api/environmentvariablevalues?$select=value&$expand=EnvironmentVariableDefinitionId&$filter=(EnvironmentVariableDefinitionId/schemaname eq '" + variableName + "')",
                    contentType: "application/json",
                    success: function (result, status, xhr) {
                        result.expirationDate = _generateExpirationDate(); 	
                        resolve(result);					
                    },
                    error: function(request, status, error) {
                        handleAjaxError(request, status, error);
                        reject(error);
                    }
                });
            });	
        };

        var _getApiToken = function() {   
            return new Promise(function (resolve, reject) {
				 var apiTokenKey = "apiToken"
                 var apiToken = JSON.parse(window.sessionStorage.getItem(apiTokenKey));	;
                 if (apiToken) {	
                       var currentDatePlusOneMinute = _generateCurrentDatePlusOneMinute();
                       if(currentDatePlusOneMinute.getTime() <= new Date(apiToken.eyfrcc_expirationdate).getTime() ){   
                            resolve(apiToken.eyfrcc_token);	 
                       }
                       else {
                            _getApiTokenFromApi()  
                            .then(value => {
                                window.sessionStorage.setItem(apiTokenKey,  JSON.stringify(value));   
                                resolve(apiToken.eyfrcc_token); 
                            })
                            .catch(reason => reject(reason));			
                       }	
                 }
                 else {
                    _getApiTokenFromApi()  
                    .then(value => {
                        window.sessionStorage.setItem(apiTokenKey,  JSON.stringify(value));    
                        resolve(value.eyfrcc_token); 
                    })
                    .catch(reason => reject(reason));			
                 }
			});				 
        };

    
        var _getApiTokenFromApi = function() {
            return new Promise(function (resolve, reject) {
                _safeAjax({
                    type: "GET",
                    url: "/_api/eyfrcc_apitokens(e0e11ba8-92f6-eb11-94ef-000d3a5aa607)?$select=eyfrcc_token,eyfrcc_expirationdate",
                    contentType: "application/json",
                    success: function (res, status, xhr) {   
                        resolve(res); 					
                    },
                    error: function(request, status, error) {
                        handleAjaxError(request, status, error);
                        reject(error);
                    }
                });
            });	
        };
	
        var callEygaApi = async(apiCall, apiContext, payload) => {
            var results = await Promise.all([
                getEnvironmentVariable(apiCall.apiType),
                getHeaders(apiCall.displayName, apiContext)
            ]);

            return new Promise(function (resolve, reject) { 
                $.ajax({
                    type: apiCall.httpMethod,
                    url: results[0] + apiCall.apiMethod,
                    headers: results[1],
                    data: apiCall.contentType == "application/json"
                        ? JSON.stringify(payload) : payload,
                    contentType: apiCall.contentType,
                    processData: apiCall.processData,
                    xhrFields: apiCall.xhrFields,
                    success: function (response, status, request) { 
                        resolve(response);					
                    },
                    error: function(request, status, error) {
                        handleAjaxError(request, status, error);
                        reject(error);
                    }
                });
            });
        };

        var getHeaders = function(apiAction, apiContext, generateEventId = true) {
            return Promise.all([        
                _getApiToken(), 
                _createEvent(generateEventId, apiAction, apiContext.regardingId, apiContext.regardingType, apiContext.userId),
                getEnvironmentVariable("eyfrcc_APIKey")]
            )
            .then(result => _returnHeaders(result[2], result[1], apiContext.regardingId, result[0]))
            .catch(error => handleError(error));
		};

        var handleError = function(errorMessage) {
            console.log(errorMessage);
        };

        var handleAjaxError = function(request, status, error) {
            console.log(error);
        };

        var downloadPortalPageDocument = function(portalPageDocumentId) { 
            var documentRecordUrl = `/_api/eyfrcc_portalpagedocuments(${portalPageDocumentId})/eyfrcc_document/$value`;
            window.open(documentRecordUrl, "_self");
        };

        var _returnHeaders = function (apiKey, eventId, regardingId, apiToken) {
            if (eventId) {
                return { "eyga-api-key": apiKey, "EYGA-EventId": eventId, "EYGA-RecordId": regardingId, "EYGA-Authorization": apiToken };
            }

            if (regardingId) {
                return { "eyga-api-key": apiKey, "EYGA-RecordId": regardingId, "EYGA-Authorization": apiToken };
            }

            return { "eyga-api-key": apiKey, "EYGA-Authorization": apiToken };
        };

        var _createEvent = function (generateEventId, apiAction, regardingId, regardingType, userId) {
            if (generateEventId && regardingId && regardingType && userId) {
                var event = {};
                event["eyfrcc_RegardingId_" + regardingType + "@odata.bind"] = `/${getEntitySetName(regardingType)}(${regardingId})`;
                event["eyfrcc_UserId_contact@odata.bind"] = `/contacts(${userId})`;
                event["eyfrcc_name"] = apiAction;
                event["eyfrcc_source"] = 643260000; // Portal

                return new Promise(function (resolve, reject) {
                    _safeAjax({
                        type: "POST",
                        url: "/_api/eyfrcc_events",
                        contentType: "application/json",
                        data: JSON.stringify(event),
                        success: function (res, status, xhr) {
                            var eventId = xhr.getResponseHeader("entityid");
                            resolve(eventId);				
                        },
                        error: function(request, status, error) {
                            handleAjaxError(request, status, error);
                            reject(error);
                        }
                    });
                });	
            }
            else {
                Promise.resolve();
            }
        };

        var getEntitySetName = (logicalName) => {
            if (!logicalName) { return logicalName; }
            if (logicalName.endsWith('s')) { return logicalName + 'es'; }
            if (logicalName.endsWith('y')) { return logicalName + 'ies'; }
            return logicalName + 's';
        };

        var handleRiskAssessmentOutcomes = async () => {
            var urlParams = new URLSearchParams(window.location.search);
            var parentAppId = urlParams.get("parentApplicationId");
            var childAppId = urlParams.get("formId");

            var outcomes = await _getRiskAssessmentOutcomes(childAppId, parentAppId);

            _handleAddFieldToApplicationOutcomes(outcomes);
            // Add code for any new logic for risk assessment outcomes here
        };

        var _handleAddFieldToApplicationOutcomes = function (outcomes) {
            var accordion = $(".accordian-title:contains('" + StepNames.RiskIndicator + "')").closest(".accordian");
            if (!accordion.length || !accordion.attr("id")) { return; }

            var contentId = accordion.attr("id").slice(0, -10);

            outcomes
                .filter(outcome => outcome.eyfrcc_outcometype === RiskAssessmentOutcomeType.AddFieldToApplication
                    && outcome.outcomefieldlogicalname)
                .forEach(outcome => {
                    var row = $(["#", contentId, " #" , outcome.outcomefieldlogicalname].join("")).closest("tr");
                    if (!row) { return; }

                    if (outcome.applicationoutcomeid) {
                        row.removeClass("hidden");
                    } else {
                        row.addClass("hidden");
                    }
                })
        };

        var _getRiskAssessmentOutcomes = function (childAppId, parentAppid) {
            return new Promise(function (resolve, reject) {
                _safeAjax({
                    type: "GET",
                    url: ["/Services/GetRiskAssessmentOutcomesByApplication/?formId=", childAppId ,"&parentApplicationId=", parentAppid].join(""),
                    contentType: "application/json",
                    success: function (res, status, xhr) {
                        res = JSON.parse(res);
                        resolve(res ? res.results : res);
                    },
                    error: function (request, status, error) {
                        handleAjaxError(request, status, error);
                        reject(error);
                    }
                });
            });	
        };

        var showProgressIndicator = (message) => {
            $("span#progressIndicatorText").text(message || "Loading...");
            $("div#progressIndicator").show();
        };

        var hideProgressIndicator = () => {
            $("div#progressIndicator").hide();
        };

        var viewMessage = function (id, messagesData) {
            if (!messagesData) { return; }

            var message = messagesData[id];
            if (!message) { return false; }

            $("#eyga-view-message-modal .modal-title").text(message.Subject || "No subject");
            $("#eyga-view-message-modal .modal-body").html(message.Description);
            $("#eyga-view-message-modal").modal("show");
            return false;
        };

        var showCreateMessageModal = async function (regardingId, regardingType, userId) {
            $("#eyga-create-message-modal").modal();
            $("#eyga-create-message-loading").show();
            $("#eyga-create-message-form").css("visibility", "hidden");

            const categorySelect = $("#eyga-create-message-file-category");

            _safeAjax({
                type: "GET",
                url: ["/Services/GetDocumentCategoriesForRecord/?regardingType=", regardingType, "&regardingId=", regardingId].join(""),
                contentType: "application/json",
                success: function (response, status, xhr) {
                    response = JSON.parse(response);

                    categorySelect.empty();
                    categorySelect.append("<option 'Other'>Other</option>");
                    response.Categories.forEach(category => {
                        if (category == "Other") { return; }
                        categorySelect.append(`<option value='${category}'>${category}</option>`);
                    });
                    categorySelect.val("Other");

                    $("#eyga-create-message-submit").off("click").click(() =>
                        _createMessageAndUploadFile(response.DocumentPath, response.Tag, regardingId, regardingType, userId));

                    $("#eyga-create-message-loading").hide();
                    $("#eyga-create-message-form").css("visibility", "visible");
                },
                error: function (request, status, error) {
                    handleAjaxError(request, status, error);
                }
            });
        };

        var _createMessageAndUploadFile = async function (documentPath, tag, regardingId, regardingType, userId) {
            showProgressIndicator("Creating message...");

            try {
                await _safeAjax({
                    type: "POST",
                    url: "/_api/eyfrcc_messages",
                    contentType: "application/json",
                    data: JSON.stringify({
                        [`eyfrcc_RegardingRecordId_${regardingType}@odata.bind`]: `/${getEntitySetName(regardingType)}(${regardingId})`,
                        ["eyfrcc_name"]: $("#eyga-create-message-subject").val(),
                        ["eyfrcc_content"]: $("#eyga-create-message-content").val(),
                        ["eyfrcc_direction"]: 643260000 // Incoming
                    }),
                    error: function (request, status, error) {
                        handleAjaxError(request, status, error);
                    }
                });

                const file = $("#eyga-create-message-file-input").prop('files')?.[0];
                if (file) {
                    const folderName = $("#eyga-create-message-file-category").find(":selected").val() || "Other";

                    var payload = new FormData();
                    payload.append('json', JSON.stringify({
                        "Name": file.name,
                        "DocumentFolder": folderName,
                        "RecordFolder": documentPath,
                        "Tags": {
                            "Id": tag,
                            "Folder": folderName,
                            "Upload Date": new Date().toLocaleString().replace(/,/g, '').replace(/:/g, '-'),
                            "Status": "Accepted"
                        },
                        "OverwriteExisting": false
                    }));
                    payload.append('file', file);

                    var eygaApiContext = new EygaApiContext(regardingId, regardingType, userId);
                    await callEygaApi(ApiCall.UploadDocument, eygaApiContext, payload);
                }
            } catch (error) {
                handleError(error);
            }

            hideProgressIndicator();
            $("#eyga-create-message-modal").modal("hide");
			$("#eyga-create-message-subject").val(null);
            $("#eyga-create-message-content").val(null);
			location.reload();
        };

		var openConfirmationModal = function(title = "Confirm", message = "Are you sure?",
			confirmLabel = "Confirm", cancelLabel = "Cancel") {
			return new Promise(function (resolve, reject) {
				$("#eyga-confirm-modal").attr("aria-label", title);
				$("#eyga-confirm-modal .modal-title").attr("title", title);
				$("#eyga-confirm-modal .modal-title").text(title);
				$("#eyga-confirm-modal .modal-body").text(message);
				
				const confirmModal = $("#eyga-confirm-modal");
				const confirmButton = $("#eyga-confirm-modal button.primary");
				const closeButtons = $("#eyga-confirm-modal button.cancel, #eyga-confirm-modal button.form-close");

                confirmButton.text(confirmLabel);
				confirmButton.click(() => {
					confirmModal.modal("hide");
					confirmButton.off("click");
					closeButtons.off("click");
					resolve(true);
				});

                $("#eyga-confirm-modal button.cancel").text(cancelLabel);
				closeButtons.click(() => {
					confirmModal.modal("hide");
					confirmButton.off("click");
					closeButtons.off("click");
					resolve(false);
				});
				
				confirmModal.modal();
			});
		};

        var insertSpanElements = function(selectors) {
            selectors.forEach(selector => {
                $(selector).after('<span class="ui"></span>');
            });
        };

        return {
            openAccordian: openAccordian,
			searchForError: searchForError,
            appAjax: appAjax,
            showIEWarning: showIEWarning,
            formatDate: formatDate,
            getEnvironmentVariable: getEnvironmentVariable,
            getHeaders: getHeaders,
            handleError: handleError,
            handleAjaxError: handleAjaxError,
            callEygaApi: callEygaApi,
            saveSession: saveSession,
            getSession: getSession,
            handleTablePaging: handleTablePaging,
            downloadPortalPageDocument: downloadPortalPageDocument,
            handleRiskAssessmentOutcomes: handleRiskAssessmentOutcomes,
            ApiCall: ApiCall,
            EygaApiContext: EygaApiContext,
            EygaApiCall: EygaApiCall,
            showProgressIndicator: showProgressIndicator,
            hideProgressIndicator: hideProgressIndicator,
            openConfirmationModal: openConfirmationModal,
            viewMessage: viewMessage,
            showCreateMessageModal: showCreateMessageModal,
            insertSpanElements: insertSpanElements
        }
    })();

    global.FRCC.Portal.Notification = (function() {
        var $processingMsgEl = $('#processingMsg'),
        _msg = 'Processing...',
        _stack = 0,
        _endTimeout;

        var show = function(msg) {
            $processingMsgEl.text(msg || _msg);
            if (_stack === 0) {
            clearTimeout(_endTimeout);
            $processingMsgEl.show();
            }
            _stack++;
        };

        var hide = function() {
            _stack--;
            if (_stack <= 0) {
                _stack = 0;
                clearTimeout(_endTimeout);
                _endTimeout = setTimeout(function () {
                    $processingMsgEl.hide();
                }, 500);
            }
        };

        return {
            show: show,
            hide: hide,
        }
    })();

})(this);